@@ -16,7 +16,7 @@ use strict;
 use warnings;
 no warnings 'once';
 
-our $VERSION = '0.009003';
+our $VERSION = '0.009004';
 $VERSION = eval $VERSION;
 
 use Carp ();
@@ -227,8 +227,11 @@ sub _find_sig {
 
 sub _warn {
   local $SIG{__WARN__};
+  return warn @_
+    if our $warn_deep;
   my @convert = _convert(@_);
   if (my $sig = _find_sig($OLD_SIG{__WARN__})) {
+    local $warn_deep = 1;
     (\&$sig)->(ref $convert[0] ? $convert[0] : join('', @convert));
   }
   else {
@@ -237,10 +240,14 @@ sub _warn {
     warn @convert;
   }
 }
+
 sub _die {
   local $SIG{__DIE__};
+  return
+    if our $die_deep;
   my @convert = _convert(@_);
   if (my $sig = _find_sig($OLD_SIG{__DIE__})) {
+    local $die_deep = 1;
     (\&$sig)->(ref $convert[0] ? $convert[0] : join('', @convert));
   }
   @convert = _ex_as_strings(@convert) if _can_stringify;
